@@ -24,10 +24,12 @@
 #define CONFIG_H
 
 #include "version.h"
-
+#include "netio.h"
 
 /* Set your charset here. ISO-8859-1 is default. */
+#ifndef TARGET_CHARSET
 #define TARGET_CHARSET "ISO-8859-1"
+#endif
 
 struct feed {
 	char *feedurl;							/* Non hashified URL */
@@ -38,6 +40,8 @@ struct feed {
 	char *description;
 	char *lastmodified;						/* Content of header as sent by the server. */
 	int lasthttpstatus;
+	netio_error_type netio_error;			/* See netio.h */
+	int connectresult;						/* Socket errno */
 	char *cookies;							/* Login cookies for this feed. */
 	char *authinfo;							/* HTTP authinfo string. */
 	char *servauth;							/* Server supplied authorization header. */
@@ -69,6 +73,7 @@ struct newsdata {
 struct keybindings {
 	char next;
 	char prev;
+	char prevmenu;
 	char quit;
 	char addfeed;
 	char deletefeed;
