@@ -31,6 +31,9 @@
 #include <unistd.h>
 #include <errno.h>
 #include <zlib.h>
+//#ifdef __STDC_ISO_10646__
+#include <libxml/HTMLparser.h>
+//#endif
 
 #include "conversions.h"
 #include "config.h"
@@ -40,23 +43,24 @@
 
 extern struct entity *first_entity;
 
+#ifdef STATIC_CONST_ICONV
+char * iconvert (const char * inbuf, char * from, char * to) {
+#else
 char * iconvert (char * inbuf, char * from, char * to) {
+#endif
 	iconv_t cd;							/* Iconvs conversion descriptor. */
 	char *outbuf, *outbuf_first;		/* We need two pointers so we do not lose
 	                                       the strings starting position. */
 	size_t inbytesleft, outbytesleft;
 
-	/* Take shortcut if TARGET_CHARSET is UTF-8 and avoid
-	   unneccessary conversion. */
-	/*
-	Something like:
-	if (strcasecmp (to, TARGET_CHARSET) == 0)
+	/* Take a shortcut. */
+	if (strcasecmp (from, to) == 0)
 		return strdup(inbuf);
-	*/
 	
 	inbytesleft = strlen(inbuf);
 	outbytesleft = strlen(inbuf);
-	
+
+	/* iconv_open(nl_langinfo(CODESET), from); */
 	cd = iconv_open (to, from);
 	if (cd == (iconv_t) -1) {
 		return NULL;
@@ -94,6 +98,14 @@ char * UIDejunk (char * feed_description) {
 	char *entity;				/* Which HTML crap did we hit now? */
 	struct entity *cur_entity;
 	int found = 0;				/* User defined entity matched? */
+#ifdef __STDC_ISO_10646__
+	wchar_t ch;					/* Decoded numeric entity */
+	int len;					/* Current length of detag'd text */
+	int mblen;					/* Length of multi-byte entity */
+#else
+	unsigned long ch;			/* Decoded numeric entity */
+#endif
+	const htmlEntityDesc *ep;	/* For looking up HTML entities */
 	
 	/* Gracefully handle passed NULL ptr. */
 	if (feed_description == NULL) {
@@ -220,6 +232,41 @@ char * UIDejunk (char * feed_description) {
 					}
 				}
 								
+				/* Try to parse some standard entities. */
+				if (!found) {
+					/* See if it was a numeric entity. */
+					if (entity[0] == '#') {
+						if (entity[1] == 'x')
+							ch = strtoul(entity + 2, NULL, 16);
+						else
+							ch = atol(entity + 1);
+					}
+					else if ((ep = htmlEntityLookup(entity)) != NULL)
+						ch = ep->value;
+					else
+						ch = 0;  /* Didn't recognize it. */
+
+					if (ch > 0) {
+#ifdef __STDC_ISO_10646__
+						/* Convert to locale encoding and append. */
+						len = strlen(newtext);
+						newtext = realloc(newtext, len + MB_CUR_MAX + 1);
+						mblen = wctomb(newtext + len, ch);
+						/* Only set found flag if the conversion worked. */
+						if (mblen > 0) {
+							newtext[len + mblen] = '\0';
+							found = 1;
+						}
+#else
+						/* Since we can't use wctomb(), just convert ASCII. */
+						if (ch <= 0x7F) {
+							sprintf(newtext + strlen(newtext), "%c", (int)ch);
+							found = 1;
+						}
+#endif
+					}
+				}
+
 				/* If nothing matched so far, put text back in. */
 				if (!found) {
 					/* Changed into &+entity to avoid stray semicolons
@@ -345,9 +392,7 @@ char * WrapText (char * text, int width) {
 		}
 	}
 	
-	//if (line != NULL)
-		free (line);
-	
+	free (line);	
 	free (start);	
 	
 	return newtext;
@@ -396,7 +441,7 @@ void *zlib_uncompress(void *in_buf, int in_size, int *out_size, int voodoo_magic
 			inflateEnd(&stream);
 			free(out_buf);
 			snprintf (tmpstring, sizeof(tmpstring), _("ERROR: zlib_uncompress: %d %s\n"), result, stream.msg);
-			UIStatus (tmpstring, 2);
+			UIStatus (tmpstring, 2, 1);
 			return NULL;
 		}
 		if (stream.avail_out < sizeof tmp_buf) {
@@ -411,7 +456,7 @@ void *zlib_uncompress(void *in_buf, int in_size, int *out_size, int voodoo_magic
 			/* For some reason, inflate() didn't write out a single byte. */
 			inflateEnd(&stream);
 			free(out_buf);
-			UIStatus (_("ERROR: No output during decompression"), 2);
+			UIStatus (_("ERROR: No output during decompression"), 2, 1);
 			return NULL;
 		}
 	} while (result != Z_STREAM_END);
@@ -444,18 +489,18 @@ void *gzip_uncompress(void *in_buf, int in_size, int *out_size) {
 		*out_size = 0;
 
 	if ((header->magic[0] != 0x1F) || (header->magic[1] != 0x8B)) {
-		UIStatus (_("ERROR: Invalid magic bytes for GZIP data"), 2);
+		UIStatus (_("ERROR: Invalid magic bytes for GZIP data"), 2, 1);
 		return NULL;
 	}
 	
 	if (header->method != 8) {
-		UIStatus (_("ERROR: Compression method is not deflate"), 2);
+		UIStatus (_("ERROR: Compression method is not deflate"), 2, 1);
 		return NULL;
 	}
 	
 	if (header->flags != 0 && header->flags != 8) {
-		snprintf (tmpstring, sizeof(tmpstring), _("ERROR: Unsupported flags %d\n"), header->flags);
-		UIStatus (tmpstring, 2);
+		snprintf (tmpstring, sizeof(tmpstring), _("ERROR: Unsupported flags %d"), header->flags);
+		UIStatus (tmpstring, 2, 1);
 		return NULL;
 	}
 	
@@ -532,6 +577,34 @@ char *base64encode(char const *inbuf, unsigned int inbuf_size) {
 	return outbuf;
 }
 
+/* Returns NULL on invalid input */
+char* decodechunked(char * chunked, unsigned int *inputlen) {
+	char *orig = chunked, *dest = chunked;
+	unsigned long chunklen;
+
+	/* We can reuse the same buffer to dechunkify it:
+	 * the data size will never increase. */
+	while((chunklen = strtoul(orig, &orig, 16))) {
+		/* process one more chunk: */
+		/* skip chunk-extension part */
+		while(*orig && (*orig != '\r'))
+			orig++;
+		/* skip '\r\n' after chunk length */
+		orig += 2;
+		if(( chunklen > (chunked + *inputlen - orig)))
+			/* insane chunk length. Well... */
+			return NULL;
+		memmove(dest, orig, chunklen);
+		dest += chunklen;
+		orig += chunklen;
+		/* and go to the next chunk */
+	}
+	*dest = '\0';
+	*inputlen = dest - chunked;
+	
+	return chunked;
+}
+
 /* Remove leading whitspaces, newlines, tabs.
  * This function should be safe for working on UTF-8 strings.
  * tidyness: 0 = only suck chars from beginning of string
