@@ -39,6 +39,8 @@
 #include "xmlparse.h"
 #include "filters.h"
 #include "dialog.h"
+#include "io-internal.h"
+#include "conversions.h"
 
 extern char *browser;
 extern struct keybindings keybindings;
@@ -69,10 +71,13 @@ char * UIOneLineEntryField (int x, int y) {
 	return text;
 }
 
-int UIChangeBrowser (void) {
+void UIChangeBrowser (void) {
 	char *browserstring;
+	char *fmtstr;
 	int len;
 	
+	repeat:
+	
 	/* malloc = 17 (strlen("Current setting: ") + browser
 	   We will malloc a bigger junk, because other languages
 	   might need longer strings and crash! */
@@ -84,22 +89,31 @@ int UIChangeBrowser (void) {
 	attron (WA_REVERSE);
 	UISupportDrawBox (3, 5, COLS-4, 7);
 		
-	UIStatus (browserstring, 0);
+	UIStatus (browserstring, 0, 0);
 	free (browserstring);
 	
 	browserstring = UIOneLineEntryField (5, 6);
 	
 	if (strlen(browserstring) == 0) {
 		free (browserstring);
-		return 1;
+		UIStatus (_("Aborted."), 1, 0);
+		return;
 	}
 	
+	fmtstr = strstr (browserstring, "%");
+	if ((fmtstr != NULL) && (fmtstr[1] != 's')) {
+		free (browserstring);
+		UIStatus (_("Only %s format strings allowed for default browser!"), 3, 1);
+		goto repeat;
+	}
+	
+	if (strstr(browserstring, "'") != NULL)
+		UIStatus (_("Unsafe browser string (contains quotes)! See snownews.kcore.de/faq#toc2.4"), 3, 1);
+	
 	browser = realloc (browser, strlen(browserstring)+1);
 	strncpy (browser, browserstring, strlen(browserstring)+1);
 	
 	free (browserstring);
-	
-	return 0;
 }
 
 /* Dialog to change feedname.
@@ -107,27 +121,28 @@ int UIChangeBrowser (void) {
            1	on user abort
 		   2	original title restored
 */
-int UIChangeFeedName (struct feed *cur_ptr) {
+void UIChangeFeedName (struct feed *cur_ptr) {
 	char *newname;
 
 	/* Clear screen area we want to "draw" to. */
 	attron (WA_REVERSE);
 	UISupportDrawBox (3, 5, COLS-4, 7);
 
-	UIStatus (_("Enter new name. Blank line to abort. '-' to reset."), 0);
+	UIStatus (_("Enter new name. Blank line to abort. '-' to reset."), 0, 0);
 	
 	newname = UIOneLineEntryField (5, 6);
 	
 	/* If strlen is zero, return 1. */
 	if (strlen(newname) == 0) {
 		free (newname);
-		return 1;
+		return;
 	}
 	
 	/* If newname contains "|", abort since this is used as a delimiter for the config file. */
 	if (strstr (newname, "|") != NULL) {
 		free (newname);
-		return 3;
+		UIStatus (_("The new title must not contain a \"|\" character!"), 2, 0);
+		return ;
 	}
 	
 	/* Restor original title. */
@@ -142,7 +157,7 @@ int UIChangeFeedName (struct feed *cur_ptr) {
 			free (cur_ptr->override);
 			cur_ptr->override = NULL;
 			free (newname);
-			return 2;
+			return;
 		}
 	}
 	
@@ -159,7 +174,7 @@ int UIChangeFeedName (struct feed *cur_ptr) {
 	cur_ptr->title = strdup (newname);
 		
 	free (newname);
-	return 0;
+	return;
 }
 
 /* Popup window to add new RSS feed. Passing an URL will
@@ -174,7 +189,7 @@ int UIAddFeed (char * newurl) {
 		attron (WA_REVERSE);
 		UISupportDrawBox (3, 5, COLS-4, 7);
 
-		UIStatus (_("Enter URL of the feed you want to add. Blank line to abort."), 0);
+		UIStatus (_("Enter URL of the feed you want to add. Blank line to abort."), 0, 0);
 
 		url = UIOneLineEntryField (5, 6);
 
@@ -184,7 +199,13 @@ int UIAddFeed (char * newurl) {
 			free (url);
 			return 1;
 		}
-
+		
+		CleanupString(url, 0);
+		
+		/* Support that stupid feed:// "protocol" */
+		if (strncasecmp (url, "feed://", 7) == 0)
+			memcpy (url, "http", 4);
+		
 		/* If URL does not start with the procotol specification,
 		assume http://
 		-> tmp[512] -> we can "only" use max 504 chars from url ("http://" == 7). */
@@ -214,6 +235,8 @@ int UIAddFeed (char * newurl) {
 	new_ptr->description = NULL;
 	new_ptr->lastmodified = NULL;
 	new_ptr->lasthttpstatus = 0;
+	new_ptr->netio_error = NET_ERR_OK;
+	new_ptr->connectresult = 0;
 	new_ptr->cookies = NULL;
 	new_ptr->authinfo = NULL;
 	new_ptr->servauth = NULL;
@@ -246,7 +269,7 @@ int UIAddFeed (char * newurl) {
 
 	/* Download new feed and DeXMLize it. */	
 	if ((UpdateFeed (new_ptr)) != 0) {
-		UIStatus (_("The feed could not be parsed. Do you need a filter script for this feed? (y/n)"), 0);
+		UIStatus (_("The feed could not be parsed. Do you need a filter script for this feed? (y/n)"), 0, 0);
 		
 		if (getch() == 'n')
 			return -1;
@@ -359,7 +382,7 @@ void FeedInfo (struct feed * current_feed) {
 		mvaddnstr (13, 7+strlen(_("Filtered through:"))+1, current_feed->perfeedfilter, COLS-14);
 	}
 	
-	UIStatus (_("Displaying feed information."), 0);
+	UIStatus (_("Displaying feed information."), 0, 0);
 	
 	free (file);
 	free (hashme);
@@ -377,7 +400,7 @@ int UIDeleteFeed (char * feedname) {
 	mvaddstr (6, COLS/2-21, _("Are you sure you want to delete this feed?"));
 	mvprintw (7, 5, "%s", feedname);
 	
-	UIStatus (_("Type 'y' to delete, any other key to abort."), 0);
+	UIStatus (_("Type 'y' to delete, any other key to abort."), 0, 0);
 	
 	if (getch() == 'y')
 		return 1;
@@ -439,7 +462,7 @@ void UIHelpScreen (void) {
 	mvaddstr (centery+9, centerx-offsetstr, _("Quit program"));
 	attroff (WA_REVERSE);
 
-	UIStatus (_("Press the any(tm) key to exit help screen."), 0);
+	UIStatus (_("Press the any(tm) key to exit help screen."), 0, 0);
 	userinput = getch();
 	
 	/* Return input back into input queue so it gets automatically
@@ -471,7 +494,7 @@ void UIDisplayFeedHelp (void) {
 	mvprintw (centery+2, centerx-offset, "%c:", keybindings.markunread);
 	mvprintw (centery+3, centerx-offset, "%c:", keybindings.feedinfo);
 	mvaddstr (centery+4, centerx-offset, _("tab:"));
-	mvprintw (centery+5, centerx-offset, "%c:", keybindings.quit);
+	mvprintw (centery+5, centerx-offset, "%c:", keybindings.prevmenu);
 	/* Descriptions */
 	mvprintw (centery-5, centerx-offsetstr, _("Previous item"));	
 	mvprintw (centery-4, centerx-offsetstr, _("Next item"));
@@ -486,7 +509,7 @@ void UIDisplayFeedHelp (void) {
 	mvprintw (centery+5, centerx-offsetstr, _("Return to main menu"));
 	attroff (WA_REVERSE);
 
-	UIStatus (_("Press the any(tm) key to exit help screen."), 0);
+	UIStatus (_("Press the any(tm) key to exit help screen."), 0, 0);
 	userinput = getch();
 	if ((userinput != '\n') && (userinput != 'h'))
 		ungetch(userinput);
@@ -509,7 +532,7 @@ void UIDisplayItemHelp (void) {
 	mvprintw (centery-1, centerx-offset, "%c, <-:", keybindings.prev);	
 	mvprintw (centery,   centerx-offset, "%c, ->:", keybindings.next);
 	mvprintw (centery+1, centerx-offset, "%c:", keybindings.urljump);
-	mvprintw (centery+2, centerx-offset, _("%c, enter:"), keybindings.quit);
+	mvprintw (centery+2, centerx-offset, _("%c, enter:"), keybindings.prevmenu);
 	/* Descriptions */
 	mvaddstr (centery-2, centerx-offsetstr, _("Scroll text"));
 	mvaddstr (centery-1, centerx-offsetstr, _("Previous item"));	
@@ -518,7 +541,7 @@ void UIDisplayItemHelp (void) {
 	mvaddstr (centery+2, centerx-offsetstr, _("Return to overview"));
 	attroff (WA_REVERSE);
 
-	UIStatus (_("Press the any(tm) key to exit help screen."), 0);
+	UIStatus (_("Press the any(tm) key to exit help screen."), 0, 0);
 	userinput = getch();
 	if ((userinput != '\n') && (userinput != 'h'))
 		ungetch(userinput);
@@ -552,10 +575,7 @@ void CategorizeFeed (struct feed * current_feed) {
 			n++;
 		}
 
-		//if (n == 0)
-			UISupportDrawBox ((COLS/2)-37, 2, (COLS/2)+37, 2+3+nglobal+1);
-		//else
-		//	UISupportDrawBox ((COLS/2)-37, 2, (COLS/2)+37, 2+3+n+1);
+		UISupportDrawBox ((COLS/2)-37, 2, (COLS/2)+37, 2+3+nglobal+1);
 		
 		attron (WA_REVERSE);
 		mvprintw (3, (COLS/2)-((strlen(_("Category configuration for %s"))+strlen(current_feed->title))/2),
@@ -576,7 +596,7 @@ void CategorizeFeed (struct feed * current_feed) {
 					i += 39;
 			}
 			snprintf (tmp, sizeof(tmp), _("Select category number to add, press 'A' to add a new category or '%c' to quit."), keybindings.quit);
-			UIStatus (tmp, 0);
+			UIStatus (tmp, 0, 0);
 		} else {
 			i = 49;
 			y = 5;
@@ -590,7 +610,7 @@ void CategorizeFeed (struct feed * current_feed) {
 					i += 39;
 			}
 			snprintf (tmp, sizeof(tmp), _("Select a category number to delete, press 'A' to add a new one or '%c' to quit."), keybindings.quit);
-			UIStatus (tmp, 0);
+			UIStatus (tmp, 0, 0);
 		}
 		
 		refresh();
@@ -616,7 +636,7 @@ void CategorizeFeed (struct feed * current_feed) {
 						i += 39;
 				}
 				snprintf (tmp, sizeof(tmp), _("Select category number to add, press 'A' to add a new category or '%c' to quit."), keybindings.quit);
-				UIStatus (tmp, 0);
+				UIStatus (tmp, 0, 0);
 				
 				uiinput = getch();
 				
@@ -627,15 +647,15 @@ void CategorizeFeed (struct feed * current_feed) {
 				UISupportDrawBox ((COLS/2)-37, 5, (COLS/2)+37, 2+3+nglobal+1);
 
 				attron (WA_REVERSE);
-				UIStatus (_("Enter new category name."), 0);
+				UIStatus (_("Enter new category name."), 0, 0);
 				newcategory = UIOneLineEntryField ((COLS/2)-33, 5);
 				if (strlen(newcategory) == 0)
-					UIStatus (_("Aborted."), 1);
+					UIStatus (_("Aborted."), 1, 0);
 				else {
 					if (!FeedCategoryExists (current_feed, newcategory))
 						FeedCategoryAdd (current_feed, newcategory);
 					else
-						UIStatus (_("Category already defined for this feed!"), 1);
+						UIStatus (_("Category already defined for this feed!"), 1, 0);
 				}
 				free (newcategory);
 			} else {
@@ -682,12 +702,6 @@ void CategorizeFeed (struct feed * current_feed) {
 	}
 }
 
-/* This function lets the user assign color labels to a category. */
-void CatConf (void) {
-	UIStatus ("Please write me! -- CatConf() 2003 A.D.", 0);
-	getch();
-}
-
 /* Allocates and returns a filter string the user has chosen from the list of
    available categories.
    If no filter is chosen a NULL pointer is returned. */
@@ -719,7 +733,7 @@ char * DialogGetCategoryFilter (void) {
 			i += 39;
 	}
 			
-	UIStatus (_("Select a category number or any other key to reset filters."), 0);
+	UIStatus (_("Select a category number or any other key to reset filters."), 0, 0);
 	
 	refresh();
 		
@@ -757,7 +771,7 @@ int UIPerFeedFilter (struct feed * current_feed) {
 	attron (WA_REVERSE);
 	UISupportDrawBox (3, 5, COLS-4, 7);
 
-	UIStatus (_("Enter script to pipe feed through."), 0);
+	UIStatus (_("Enter script to pipe feed through."), 0, 0);
 	
 	newstring = UIOneLineEntryField (5, 6);
 	
