@@ -28,13 +28,12 @@ void UIDisplayFeedHelp (void);
 void UIDisplayItemHelp (void);
 void UIAbout (void);
 char * UIOneLineEntryField (int x, int y);
-int UIChangeBrowser (void);
-int UIChangeFeedName (struct feed *cur_ptr);
+void UIChangeBrowser (void);
+void UIChangeFeedName (struct feed *cur_ptr);
 int UIAddFeed (char * newurl);
 void FeedInfo (struct feed * current_feed);
 int UIDeleteFeed (char * feedname);
 void CategorizeFeed (struct feed * current_feed);
-void CatConf (void);
 char * DialogGetCategoryFilter (void);
 int UIPerFeedFilter (struct feed * current_feed);
 
