@@ -23,6 +23,10 @@
  *
  */
 
+#ifdef UTF_8
+#define xmlStrlen xmlUTF8Strlen
+#endif
+
 #include <ncurses.h>
 
 #include <string.h>
@@ -40,6 +44,8 @@
 #include "ui-support.h"
 #include "categories.h"
 #include "about.h"
+#include "io-internal.h"
+#include "os-support.h"
 
 extern struct keybindings keybindings;
 extern struct color color;
@@ -252,15 +258,15 @@ void UIDisplayItem (struct newsitem * current_item, struct feed * current_feed,
 		current_item->data->readstatus = 1;
 		
 		snprintf (tmpstr, sizeof(tmpstr), _("Press '%c' or Enter to return to previous screen. Hit '%c' for help screen."),
-			keybindings.quit, keybindings.help);
-		UIStatus (tmpstr, 0);
+			keybindings.prevmenu, keybindings.help);
+		UIStatus (tmpstr, 0, 0);
 		uiinput = getch();
 		
 		if ((uiinput == keybindings.help) ||
 			(uiinput == '?'))
 			UIDisplayItemHelp();
 		if ((uiinput == '\n') ||
-			(uiinput == keybindings.quit) || (uiinput == keybindings.enter)) {
+			(uiinput == keybindings.prevmenu) || (uiinput == keybindings.enter)) {
 			/* Free the wrapped text linked list.
 			   Why didn't valgrind find this? Counted as "still reachable".
 			   Strange voodoo magic may be going on here! Maybe we just append
@@ -289,7 +295,7 @@ void UIDisplayItem (struct newsitem * current_item, struct feed * current_feed,
 			} else {
 				/* Setting rewrap to 1 to get the free block below executed. */
 				rewrap = 1;
-				uiinput = ungetch(keybindings.quit);
+				uiinput = ungetch(keybindings.prevmenu);
 			}
 		}
 		if ((uiinput == keybindings.prev) ||
@@ -302,7 +308,7 @@ void UIDisplayItem (struct newsitem * current_item, struct feed * current_feed,
 			} else {
 				/* Setting rewrap to 1 to get the free block below executed. */
 				rewrap = 1;
-				uiinput = ungetch(keybindings.quit);
+				uiinput = ungetch(keybindings.prevmenu);
 			}
 		}
 		/* Scroll by one page. */
@@ -384,8 +390,7 @@ int UIDisplayFeed (struct feed * current_feed) {
 	struct newsitem *tmp_first;
 	struct newsitem *first_scr_ptr;
 	struct newsitem *markasread;
-	
-	
+		
 	/* Set highlighted to first_ptr at the beginning.
 	   Otherwise bad things (tm) will happen. */
 	first_scr_ptr = current_feed->items;
@@ -461,7 +466,7 @@ int UIDisplayFeed (struct feed * current_feed) {
 					/* if (strncmp (search, cur_ptr->title, searchlen) == 0) {*/
 					/* Substring match. */
 					if (cur_ptr->data->title != NULL) {
-						if (strstr (cur_ptr->data->title, search) != NULL) {
+						if (s_strcasestr (cur_ptr->data->title, search) != NULL) {
 							found = 1;
 							highlighted = cur_ptr;
 							if (skip >= 1) {
@@ -547,7 +552,7 @@ int UIDisplayFeed (struct feed * current_feed) {
 			if (curitem_ptr == highlighted) {
 				highlightnum = itemnum;
 				attron (WA_REVERSE);
-				for (i = 0; i < COLS-1; i++)
+				for (i = 0; i < COLS; i++)
 					mvaddch (ypos, i, ' ');
 			}
 			
@@ -621,11 +626,11 @@ int UIDisplayFeed (struct feed * current_feed) {
 		
 		if (typeahead) {
 			snprintf (tmpstr, sizeof(tmpstr), "-> %s", search);
-			UIStatus (tmpstr, 0);
+			UIStatus (tmpstr, 0, 0);
 		} else {
 			snprintf (tmpstr, sizeof(tmpstr), _("Press '%c' to return to main menu, '%c' to show help."),
-				keybindings.quit, keybindings.help);
-			UIStatus (tmpstr, 0);
+				keybindings.prevmenu, keybindings.help);
+			UIStatus (tmpstr, 0, 0);
 		}
 		
 		uiinput = getch();
@@ -654,7 +659,7 @@ int UIDisplayFeed (struct feed * current_feed) {
 			if ((uiinput == keybindings.help) ||
 				(uiinput == '?'))
 				UIDisplayFeedHelp();
-			if (uiinput == keybindings.quit) {
+			if (uiinput == keybindings.prevmenu) {
 				free (categories);
 				return reloaded;
 			}
@@ -730,9 +735,11 @@ int UIDisplayFeed (struct feed * current_feed) {
 			}
 			if (uiinput == keybindings.reload) {
 				if (first_bak != NULL) {
-					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
 					continue;
 				}
+				if (current_feed->smartfeed == 1)
+					continue;
 				UpdateFeed (current_feed);
 				highlighted = current_feed->items;
 				/* Reset first_scr_ptr if reloading. */
@@ -754,6 +761,7 @@ int UIDisplayFeed (struct feed * current_feed) {
 			}
 			if (uiinput == keybindings.markunread) {
 				highlighted->data->readstatus = 0;
+				reloaded = 1;
 			}
 			if (uiinput == keybindings.about)
 				UIAbout();
@@ -764,7 +772,8 @@ int UIDisplayFeed (struct feed * current_feed) {
 				forceredraw = 1;
 		}
 		
-		if ((uiinput == '\n') || (uiinput == keybindings.enter)) {
+		if ((uiinput == '\n') || 
+			((uiinput == keybindings.enter) && (!typeahead))) {
 			/* If typeahead is active clear it's state and free the structure. */
 			if (typeahead) {
 				free (search);
@@ -823,7 +832,7 @@ int UIDisplayFeed (struct feed * current_feed) {
 					for (cur_ptr = current_feed->items; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
 						/* Substring match. */
 						if (cur_ptr->data->title != NULL) {	/* Don't crash on items with no title! */
-							if (strstr (cur_ptr->data->title, search) != NULL)
+							if (s_strcasestr (cur_ptr->data->title, search) != NULL)
 								found++;
 						}
 					}
@@ -981,9 +990,6 @@ void UIMainInterface (void) {
 					}
 					if (!found)
 						continue;
-				/* Include new items feed in filtered mode. */
-				//} else if (cur_ptr->smartfeed == 1) {
-				//	found = 1;
 				} else
 					continue;
 				
@@ -1063,7 +1069,7 @@ void UIMainInterface (void) {
 					/* Exact match from beginning of line. */
 					/* if (strncmp (search, cur_ptr->title, searchlen) == 0) { */
 					/* Substring match. */
-					if (strstr (cur_ptr->title, search) != NULL) {
+					if (s_strcasestr (cur_ptr->title, search) != NULL) {
 						found = 1;
 						highlighted = cur_ptr;
 						if (skip >= 1) {
@@ -1101,22 +1107,11 @@ void UIMainInterface (void) {
 					newcount++;
 			}
 			
-			/* Doesn't work. Highlight vanishes on topmost line. ncurses bug? */
-			/* Switch on color if newcount>0 */
-			//if (newcount > 0) {
-				/* Print feed name in newitems color. */
-			//	if (use_colors) {
-			//		attron (COLOR_PAIR(2));
-			//		if (color.newitemsbold)
-			//			attron (WA_BOLD);
-			//	}
-			//}
-			
 			/* Make highlight if we are the highlighted feed */
 			if (cur_ptr == highlighted) {
 				highlightnum = itemnum;
 				attron (WA_REVERSE);
-				for (i = 0; i < COLS-1; i++)
+				for (i = 0; i < COLS; i++)
 					mvaddch (ypos, i, ' ');
 			}
 			
@@ -1134,8 +1129,6 @@ void UIMainInterface (void) {
 			
 			/* Execute this _after_ the for loop. Otherwise it'll suck CPU like crazy! */
 			if (newcount != 0) {
-				//snprintf (tmp, sizeof(tmp), _("%3d new"), newcount);
-				//mvaddstr (ypos, COLS-5-strlen(_("new")), tmp);
 				localized_msg = ngettext("%3d new", "%3d new", newcount);
 				snprintf(tmp, sizeof(tmp), localized_msg, newcount);
 				mvaddstr (ypos, COLS-1-strlen(localized_msg), tmp);
@@ -1149,16 +1142,6 @@ void UIMainInterface (void) {
 			if (cur_ptr == highlighted)
 				attroff (WA_REVERSE);
 			
-			/* Doesn't work. Highlight vanishes on topmost line. ncurses bug? */
-			/* Switch off color mode again. */
-			//if (newcount > 0) {
-			//	if (use_colors) {
-			//		attroff (COLOR_PAIR(2));
-			//		if (color.newitemsbold)
-			//			attroff (WA_BOLD);
-			//	}
-			//}
-			
 			if (itemnum >= LINES-4)
 				break;
 			itemnum++;
@@ -1166,10 +1149,10 @@ void UIMainInterface (void) {
 
 		if (typeahead) {
 			snprintf (tmp, sizeof(tmp), "-> %s", search);
-			UIStatus (tmp, 0);
+			UIStatus (tmp, 0, 0);
 		} else {
 			snprintf (tmp, sizeof(tmp), _("Press '%c' for help window."), keybindings.help);
-			UIStatus (tmp, 0);
+			UIStatus (tmp, 0, 0);
 		}
 		
 		uiinput = getch();
@@ -1202,33 +1185,17 @@ void UIMainInterface (void) {
 			}
 			if (uiinput == keybindings.reload) {
 				if (filters[0] != NULL) {
-					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
 					continue;
 				}
-				/* Refresh highlighted feed. */
-				if (UpdateFeed (highlighted) != 0) {
-					if (highlighted->lasthttpstatus == 0) {
-						UIStatus (_("Aborted."), 1);
-						continue;
-					}
-					if (highlighted->lasthttpstatus == 304)
-						continue;
-					if (highlighted->lasthttpstatus != 200) {
-						snprintf (tmp, sizeof(tmp), _("HTTP error %d - Could not update feed."), highlighted->lasthttpstatus);
-						UIStatus (tmp, 2);
-						continue;
-					}
-					if (highlighted->lasthttpstatus != 0) {
-						snprintf (tmp, sizeof(tmp), _("Could not update feed %s - %s"), highlighted->title, strerror(errno));
-						UIStatus (tmp, 2);
-					}
-				}
+				UpdateFeed (highlighted);
+				
 				forceredraw = 1;
 				update_smartfeeds = 1;
 			}
 			if (uiinput == keybindings.reloadall) {
 				if (filters[0] != NULL) {
-					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
 					continue;
 				}
 				UpdateAllFeeds();
@@ -1238,30 +1205,28 @@ void UIMainInterface (void) {
 			if ((uiinput == keybindings.addfeed) ||
 				(uiinput == keybindings.newheadlines)) {
 				if (filters[0] != NULL) {
-					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
 					continue;
 				}
 				if (uiinput == keybindings.addfeed) {
 					switch (UIAddFeed(NULL)) {
 						case 0:
-							UIStatus (_("Successfully added new item..."), 1);
-							break;
-						case 1:
-							UIStatus (_("Aborted."), 1);
-							continue;
+							UIStatus (_("Successfully added new item..."), 1, 0);
 							break;
 						case 2:
-							UIStatus (_("Invalid URL! Please add http:// if you forgot this."), 2);
+							UIStatus (_("Invalid URL! Please add http:// if you forgot this."), 2, 1);
 							continue;
 							break;
 						case -1:
-							UIStatus (_("There was a problem adding the feed!"), 2);
+							UIStatus (_("There was a problem adding the feed!"), 2, 1);
 							break;
 						default:
 							break;
 					}
-				} else
-					UIAddFeed ("smartfeed:/newitems");
+				} else {
+					if (!SmartFeedExists("newitems"))
+						UIAddFeed ("smartfeed:/newitems");
+				}
 				
 				/* Scroll to top of screen and redraw everything. */
 				highlighted = first_ptr;
@@ -1275,7 +1240,7 @@ void UIMainInterface (void) {
 			if (uiinput == keybindings.deletefeed) {
 				/* This should be moved to its own function in ui-support.c! */
 				if (filters[0] != NULL) {
-					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
 					continue;
 				}
 				
@@ -1343,36 +1308,36 @@ void UIMainInterface (void) {
 					highlighted = cur_ptr;
 				
 					/* free (removed) pointer */
-					if (removed->items != NULL) {
-						while (removed->items->next_ptr != NULL) {
-							removed->items = removed->items->next_ptr;
-							free (removed->items->prev_ptr->data->title);
-							free (removed->items->prev_ptr->data->link);
-							free (removed->items->prev_ptr->data->description);
-							free (removed->items->prev_ptr);
+					if (removed->smartfeed == 0) {
+						if (removed->items != NULL) {
+							while (removed->items->next_ptr != NULL) {
+								removed->items = removed->items->next_ptr;
+								free (removed->items->prev_ptr->data->title);
+								free (removed->items->prev_ptr->data->link);
+								free (removed->items->prev_ptr->data->description);
+								free (removed->items->prev_ptr);
+							}
+							free (removed->items->data->title);
+							free (removed->items->data->link);
+							free (removed->items->data->description);
+							free (removed->items);
 						}
-						free (removed->items->data->title);
-						free (removed->items->data->link);
-						free (removed->items->data->description);
-						free (removed->items);
+						
+						free (removed->feedurl);
+						free (removed->feed);
+						free (removed->title);
+						free (removed->link);
+						free (removed->description);
+						free (removed->lastmodified);
+						free (removed->override);
+						free (removed->original);
+						free (removed->cookies);
+						free (removed->authinfo);
+						free (removed->servauth);
+						free (removed);
 					}
 					
-					free (removed->feedurl);
-					free (removed->feed);
-					free (removed->title);
-					free (removed->link);
-					free (removed->description);
-					free (removed->lastmodified);
-					free (removed->override);
-					free (removed->original);
-					free (removed->cookies);
-					free (removed->authinfo);
-					free (removed->servauth);
-					free (removed);			
-					
 					update_smartfeeds = 1;
-				} else {
-					UIStatus (_("Aborted."), 1);
 				}
 			}
 			if ((uiinput == KEY_UP) ||
@@ -1450,7 +1415,7 @@ void UIMainInterface (void) {
 			if (uiinput == keybindings.moveup) {
 				/* This function is deactivated when a filter is active. */
 				if (filters[0] != NULL) {
-					UIStatus (_("You cannot move items while a category filter is defined!"), 2);
+					UIStatus (_("You cannot move items while a category filter is defined!"), 2, 0);
 					continue;
 				}
 				/* Move item up. */
@@ -1462,11 +1427,12 @@ void UIMainInterface (void) {
 								first_scr_ptr = first_scr_ptr->prev_ptr;
 						}
 				}
+				update_smartfeeds = 1;
 			}
 			if (uiinput == keybindings.movedown) {
 				/* This function is deactivated when a filter is active. */
 				if (filters[0] != NULL) {
-					UIStatus (_("You cannot move items while a category filter is defined!"), 2);
+					UIStatus (_("You cannot move items while a category filter is defined!"), 2, 0);
 					continue;
 				}
 				/* Move item down. */
@@ -1478,12 +1444,10 @@ void UIMainInterface (void) {
 								first_scr_ptr = first_scr_ptr->next_ptr;
 					}
 				}
+				update_smartfeeds = 1;
 			}
-			if (uiinput == keybindings.dfltbrowser) {
-				if (UIChangeBrowser()) {
-					UIStatus (_("Aborted."), 1);
-				}
-			}
+			if (uiinput == keybindings.dfltbrowser)
+				UIChangeBrowser();
 			if (uiinput == keybindings.markallread) {
 				/* This function is safe for using in filter mode, because it only
 				   changes int values. It automatically marks the correct ones read
@@ -1499,42 +1463,32 @@ void UIMainInterface (void) {
 			if (uiinput == keybindings.changefeedname) {
 				/* This needs to be worked on before it works while a filter is applied! */
 				if (filters[0] != NULL) {
-					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
 					continue;
 				}
-				switch (UIChangeFeedName(highlighted)) {
-					case 0:
-						/* Changed */
-						UIStatus (_("Title changed successfully."), 1);
-						break;
-					case 1:
-						/* Aborted */
-						UIStatus (_("Aborted."), 1);
-						break;
-					case 2:
-						UIStatus (_("Original title restored."), 1);
-						break;
-					case 3:
-						UIStatus (_("The new title must not contain a \"|\" character!"), 2);
-						break;
-					default:
-						break;
-				}
+				UIChangeFeedName(highlighted);
+				update_smartfeeds = 1;
 			}
-			if (uiinput == keybindings.perfeedfilter)
+			if (uiinput == keybindings.perfeedfilter) {
+				if (filters[0] != NULL) {
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
+					continue;
+				}
 				UIPerFeedFilter (highlighted);
+			}
 			if (uiinput == keybindings.sortfeeds) {
 				/* Deactivate sorting function if filter is applied. */
 				if (filters[0] != NULL) {
-					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
 					continue;
 				}
 				SnowSort();
+				update_smartfeeds = 1;
 			}
 			if (uiinput == keybindings.categorize) {
 				/* This needs to be worked on before it works while a filter is applied! */
 				if (filters[0] != NULL) {
-					UIStatus (_("Please deactivate the category filter before using this function."), 2);
+					UIStatus (_("Please deactivate the category filter before using this function."), 2, 0);
 					continue;
 				}
 				if (highlighted->smartfeed != 0)
@@ -1619,7 +1573,8 @@ void UIMainInterface (void) {
 			if (uiinput == 12)
 				forceredraw = 1;
 		}
-		if ((uiinput == '\n') || (uiinput == keybindings.enter)) {
+		if ((uiinput == '\n') || 
+			((uiinput == keybindings.enter) && (!typeahead))) {
 			/* If typeahead is active clear it's state and free the structure. */
 			if (typeahead) {
 				free (search);
@@ -1655,7 +1610,7 @@ void UIMainInterface (void) {
 					found = 0;
 					for (cur_ptr = first_ptr; cur_ptr != NULL; cur_ptr = cur_ptr->next_ptr) {
 						/* Substring match. */
-						if (strstr (cur_ptr->title, search) != NULL)
+						if (s_strcasestr (cur_ptr->title, search) != NULL)
 							found++;
 					}
 					if (skip == found-1) { /* found-1 to avoid empty tab cycle. */
