@@ -27,10 +27,5 @@
 
 void MainQuit(char * func, char * error);
 void Hashify (char * url);
-int UpdateFeed (struct feed * cur_ptr);
-int UpdateAllFeeds (void);
-int LoadFeed (struct feed * cur_ptr);
-int LoadAllFeeds (int numfeeds);
-void WriteCache (void);
 
 #endif
