@@ -36,6 +36,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <errno.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -44,6 +45,8 @@
 #include <netinet/in.h>
 #include <netdb.h>
 #include <sys/stat.h>
+#include <sys/time.h>
+#include <assert.h>
 
 #include <ncurses.h>
 
@@ -53,26 +56,85 @@
 #include "conversions.h"
 #include "ui-support.h"
 #include "net-support.h"
+#include "io-internal.h"
 
-#define MAX_HTTP_REDIRECTS 10	/* Maximum number of redirects we will follow. */
+static int const MAX_HTTP_REDIRECTS = 10;	/* Maximum number of redirects we will follow. */
+static int const NET_TIMEOUT = 20;			/* Global network timeout in sec */
+static int const NET_READ = 1;
+static int const NET_WRITE = 2;
 
-extern char *proxyname;				/* Hostname of proxyserver. */
-extern unsigned short proxyport;	/* Port on proxyserver to use. */
+extern char *proxyname;						/* Hostname of proxyserver. */
+extern unsigned short proxyport;			/* Port on proxyserver to use. */
 extern char *useragent;
 
+/* Waits NET_TIMEOUT seconds for the socket to return data.
+ *
+ * Returns
+ *
+ *	0	Socket is ready
+ *	-1	Error occured (netio_error is set)
+ */
+int NetPoll (struct feed * cur_ptr, int * my_socket, int rw) {
+	fd_set rfdsr;
+	fd_set rfdsw;
+	struct timeval tv;
+	int retval;				/* FD_ISSET + assert == Heisenbug? */
+	
+	/* Set global network timeout */
+	tv.tv_sec = NET_TIMEOUT;
+	tv.tv_usec = 0;
+	
+	FD_ZERO(&rfdsr);
+	FD_ZERO(&rfdsw);
+	
+	if (rw == NET_READ) {
+		FD_SET(*my_socket, &rfdsr);
+		if (select (*my_socket+1, &rfdsr, NULL, NULL, &tv) == 0) {
+			/* Timed out */
+			cur_ptr->netio_error = NET_ERR_TIMEOUT;
+			return -1;
+		}
+		retval = FD_ISSET (*my_socket, &rfdsr);
+		assert (retval);
+		if (!retval) {
+			/* Wtf? */
+			cur_ptr->netio_error = NET_ERR_UNKNOWN;
+			return -1;
+		}
+	} else if (rw == NET_WRITE) {
+		FD_SET(*my_socket, &rfdsw);
+		if (select (*my_socket+1, NULL, &rfdsw, NULL, &tv) == 0) {
+			/* Timed out */
+			cur_ptr->netio_error = NET_ERR_TIMEOUT;
+			return -1;
+		}
+		retval = FD_ISSET (*my_socket, &rfdsw);
+		assert (retval);
+		if (!retval) {
+			/* Wtf? */
+			cur_ptr->netio_error = NET_ERR_UNKNOWN;
+			return -1;
+		}
+	} else {
+		cur_ptr->netio_error = NET_ERR_UNKNOWN;
+		return -1;
+	}
+	
+	return 0;
+}
+
+
 /* Connect network sockets.
  *
- * Return codes:	1	socket creation error
- *					2	hostname resolve error
- *					3	couldn't connect
- *                  -1  aborted by user
+ * Returns
+ *
+ *	0	Connected
+ *	-1	Error occured (netio_error is set)
  */
-int NetConnect (int * my_socket, int * connectresult, char * host, int httpproto) {
-	int retval;
+int NetConnect (int * my_socket, char * host, struct feed * cur_ptr, int httpproto, int suppressoutput) {
+	char tmp[512];
 	struct sockaddr_in address;	
 	struct hostent *remotehost;
-	fd_set rfdsr;
-	fd_set rfdsw;
 	socklen_t len;
 	char *realhost;
 	unsigned short port;
@@ -82,10 +144,20 @@ int NetConnect (int * my_socket, int * connectresult, char * host, int httpproto
 		port = 80;
 	}
 	
+	if (!suppressoutput) {
+		if (cur_ptr->title == NULL)
+			snprintf (tmp, sizeof(tmp), _("Downloading \"%s\""), cur_ptr->feedurl);
+		else
+			snprintf (tmp, sizeof(tmp), _("Downloading \"%s\""), cur_ptr->title);
+
+		UIStatus (tmp, 0, 0);
+	}
+	
 	/* Create a inet stream TCP socket. */
 	*my_socket = socket (AF_INET, SOCK_STREAM, 0);
 	if (*my_socket == -1) {
-		return 1;
+		cur_ptr->netio_error = NET_ERR_SOCK_ERR;
+		return -1;
 	}
 	
 	/* Set socket to nonblock mode to make it possible to interrupt the connect. */
@@ -99,7 +171,8 @@ int NetConnect (int * my_socket, int * connectresult, char * host, int httpproto
 		if (remotehost == NULL) {
 			close (*my_socket);
 			free (realhost);
-			return 2;
+			cur_ptr->netio_error = NET_ERR_HOST_NOT_FOUND;
+			return -1;
 		}
 		
 		/* Set the remote address. */
@@ -108,49 +181,34 @@ int NetConnect (int * my_socket, int * connectresult, char * host, int httpproto
 		memcpy (&address.sin_addr.s_addr, remotehost->h_addr_list[0], remotehost->h_length);
 			
 		/* Connect socket. */
-		*connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
+		cur_ptr->connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
 		
 		/* Check if we're already connected.
 		   BSDs will return 0 on connect even in nonblock if connect was fast enough. */
-		if (*connectresult != 0) {
+		if (cur_ptr->connectresult != 0) {
 			/* If errno is not EINPROGRESS, the connect went wrong. */
 			if (errno != EINPROGRESS) {
 				close (*my_socket);
 				free (realhost);
-				return 3;
+				cur_ptr->netio_error = NET_ERR_CONN_REFUSED;
+				return -1;
 			}
 			
-			while (1) {
-				/* stdin is read, socket is write, so we need read/write sets. 
-				   select() seems to destroy &rfdsw. Put the initialization
-				   inside the while() loop. */
-				FD_ZERO(&rfdsr);
-				FD_ZERO(&rfdsw);
-				FD_SET(0, &rfdsr);
-				FD_SET(*my_socket, &rfdsw);
-	
-				retval = select (*my_socket+1, &rfdsr, &rfdsw, NULL, NULL);
-				
-				if (FD_ISSET (0, &rfdsr)) {
-					if (getch() == 'z') {
-						close (*my_socket);
-						free (realhost);
-						return -1;
-					}
-				}
-		
-				if (FD_ISSET (*my_socket, &rfdsw))
-					break;
+			if ((NetPoll (cur_ptr, my_socket, NET_WRITE)) == -1) {
+				close (*my_socket);
+				free (realhost);
+				return -1;
 			}
 			
 			/* We get errno of connect back via getsockopt SO_ERROR (into connectresult). */
-			len = sizeof(*connectresult);
-			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, connectresult, &len);
+			len = sizeof(cur_ptr->connectresult);
+			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, &cur_ptr->connectresult, &len);
 			
-			if (*connectresult != 0) {
+			if (cur_ptr->connectresult != 0) {
 				close (*my_socket);
 				free (realhost);
-				return 3;
+				cur_ptr->netio_error = NET_ERR_CONN_FAILED;	/* ->strerror(cur_ptr->connectresult) */
+				return -1;
 			}
 		}
 	} else {
@@ -159,7 +217,8 @@ int NetConnect (int * my_socket, int * connectresult, char * host, int httpproto
 		if (remotehost == NULL) {
 			close (*my_socket);
 			free (realhost);
-			return 2;
+			cur_ptr->netio_error = NET_ERR_HOST_NOT_FOUND;
+			return -1;
 		}
 		
 		/* Set the remote address. */
@@ -168,45 +227,32 @@ int NetConnect (int * my_socket, int * connectresult, char * host, int httpproto
 		memcpy (&address.sin_addr.s_addr, remotehost->h_addr_list[0], remotehost->h_length);
 		
 		/* Connect socket. */
-		*connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
+		cur_ptr->connectresult = connect (*my_socket, (struct sockaddr *) &address, sizeof(address));
 		
 		/* Check if we're already connected.
 		   BSDs will return 0 on connect even in nonblock if connect was fast enough. */
-		if (connectresult != 0) {
+		if (cur_ptr->connectresult != 0) {
 			if (errno != EINPROGRESS) {
 				close (*my_socket);
 				free (realhost);
-				return 3;
+				cur_ptr->netio_error = NET_ERR_CONN_REFUSED;
+				return -1;
 			}
 		
-			while (1) {
-				/* stdin is read, socket is write, so we need read/write sets. */
-				FD_ZERO(&rfdsr);
-				FD_ZERO(&rfdsw);
-				FD_SET(0, &rfdsr);
-				FD_SET(*my_socket, &rfdsw);
-				
-				retval = select (*my_socket+1, &rfdsr, &rfdsw, NULL, NULL);
-			
-				if (FD_ISSET (0, &rfdsr)) {
-					if (getch() == 'z') {
-						close (*my_socket);
-						free (realhost);
-						return -1;
-					}
-				}
-			
-				if (FD_ISSET (*my_socket, &rfdsw))
-					break;
-			}
-			
-			len = sizeof(*connectresult);
-			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, connectresult, &len);
-			
-			if (*connectresult != 0) {
+			if ((NetPoll (cur_ptr, my_socket, NET_WRITE)) == -1) {
 				close (*my_socket);
 				free (realhost);
-				return 3;
+				return -1;
+			}
+			
+			len = sizeof(cur_ptr->connectresult);
+			getsockopt(*my_socket, SOL_SOCKET, SO_ERROR, &cur_ptr->connectresult, &len);
+			
+			if (cur_ptr->connectresult != 0) {
+				close (*my_socket);
+				free (realhost);
+				cur_ptr->netio_error = NET_ERR_CONN_FAILED;	/* ->strerror(cur_ptr->connectresult) */
+				return -1;
 			}
 		}
 	}
@@ -227,18 +273,18 @@ int NetConnect (int * my_socket, int * connectresult, char * host, int httpproto
  * if needed).
  * Updates passed feed struct with values gathered from webserver.
  * Handles all redirection and HTTP status decoding.
- * Returns NULL pointer if no data was received. Check httpstatus == 304,
- * otherwise an error occured.
+ * Returns NULL pointer if no data was received and sets netio_error.
  */
-char * NetIO (int * my_socket, int * connectresult, char * host, char * url, struct feed * cur_ptr, char * authdata, int httpproto, int suppressoutput) {
+char * NetIO (int * my_socket, char * host, char * url, struct feed * cur_ptr, char * authdata, int httpproto, int suppressoutput) {
 	char netbuf[4096];			/* Network read buffer. */
 	char *body;					/* XML body. */
-	int length;
+	unsigned int length;
 	FILE *stream;				/* Stream socket. */
 	int chunked = 0;			/* Content-Encoding: chunked received? */
 	int redirectcount;			/* Number of HTTP redirects followed. */
 	char httpstatus[4];			/* HTTP status sent by server. */
-	char tmp[1024];
+	char tmp[256];
+	char servreply[128];		/* First line of server reply */
 	char *tmpstatus;
 	char *savestart;			/* Save start position of pointers. */
 	char *tmphost;				/* Pointers needed to strsep operation. */
@@ -248,15 +294,15 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 	char *tmpstring;			/* Temp pointers. */
 	char *freeme, *freeme2;
 	char *redirecttarget;
-	fd_set rfds;
 	int retval;
 	int handled;
 	int tmphttpstatus;
-	int inflate = 0;			/* Whether feed data needs decompressed with deflate(1), gzip(2). */
+	int inflate = 0;			/* Whether feed data needs decompressed with zlib. */
 	int len;
 	char * inflatedbody;
 	int quirksmode = 0;			/* IIS operation mode. */
 	int authfailed = 0;			/* Avoid repeating failed auth requests endlessly. */
+
 	
 	if (!suppressoutput) {
 		if (cur_ptr->title == NULL)
@@ -264,7 +310,7 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 		else
 			snprintf (tmp, sizeof(tmp), _("Downloading \"%s\""), cur_ptr->title);
 
-		UIStatus (tmp, 0);
+		UIStatus (tmp, 0, 0);
 	}
 	
 	redirectcount = 0;
@@ -288,8 +334,7 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 		   go away if we retry.
 		   
 		   BeOS will stupidly return SUCCESS here making this code silently fail on BeOS. */
-		snprintf (tmp, sizeof(tmp), "Could not connect network socket: %s", strerror(errno));
-		UIStatus (tmp, 2);
+		cur_ptr->netio_error = NET_ERR_SOCK_ERR;
 		return NULL;
 	}
 	
@@ -314,7 +359,7 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 					(cur_ptr->authinfo ? cur_ptr->authinfo : ""),
 					(cur_ptr->cookies ? cur_ptr->cookies : ""));
 		}
-		fflush(stream);
+		fflush(stream);		/* We love Solaris, don't we? */
 	} else {
 		/* Request URL from HTTP server. */
 		if (cur_ptr->lastmodified != NULL) {
@@ -337,36 +382,19 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 					(cur_ptr->authinfo ? cur_ptr->authinfo : ""),
 					(cur_ptr->cookies ? cur_ptr->cookies : ""));
 		}
-		fflush(stream);
+		fflush(stream);		/* We love Solaris, don't we? */
 	}
 	
-	while (1) {
-	/* Use select to make the connection interuptable if it should hang.
-	   select() seems to destroy &rfdsw. Put the initialization
-	   inside the while() loop. */
-		FD_ZERO(&rfds);
-		FD_SET(0, &rfds);
-		FD_SET(*my_socket, &rfds);
-	
-		retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
-		
-		if (FD_ISSET (0, &rfds)) {
-			if (getch() == 'z') {
-				fclose (stream);
-				return NULL;
-			}
-		}
-		
-		if (FD_ISSET (*my_socket, &rfds))
-			break;
-	}
-	
-	if ((fgets (tmp, sizeof(tmp), stream)) == NULL) {
+	if ((NetPoll (cur_ptr, my_socket, NET_READ)) == -1) {
 		fclose (stream);
 		return NULL;
 	}
-	tmpstatus = malloc (strlen(tmp)+1);
-	strcpy (tmpstatus, tmp);
+	
+	if ((fgets (servreply, sizeof(servreply), stream)) == NULL) {
+		fclose (stream);
+		return NULL;
+	}
+	tmpstatus = strdup(servreply);
 	savestart = tmpstatus;
 
 	memset (httpstatus, 0, 4);	/* Nullify string so valgrind shuts up. */
@@ -375,6 +403,12 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 	            ^
 	   Copy three bytes into httpstatus. */
 	strsep (&tmpstatus, " ");
+	if (tmpstatus == NULL) {
+		cur_ptr->netio_error = NET_ERR_HTTP_PROTO_ERR;
+		fclose (stream);
+		free (savestart);	/* Probably more leaks when doing auth and abort here. */
+		return NULL;
+	}
 	strncpy (httpstatus, tmpstatus, 3);
 	free (savestart);
 	
@@ -394,10 +428,11 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 		switch (tmphttpstatus) {
 			case 200:	/* OK */
 				/* Received good status from server, clear problem field. */
+				cur_ptr->netio_error = NET_ERR_OK;
 				cur_ptr->problem = 0;
 				break;
 			case 300:	/* Multiple choice and everything 300 not handled is fatal. */
-				cur_ptr->problem = 1;
+				cur_ptr->netio_error = NET_ERR_HTTP_NON_200;
 				fclose (stream);
 				return NULL;
 			case 301:
@@ -410,16 +445,16 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 			
 				/* Give up if we reach MAX_HTTP_REDIRECTS to avoid loops. */
 				if (redirectcount > MAX_HTTP_REDIRECTS) {
+					cur_ptr->netio_error = NET_ERR_REDIRECT_COUNT_ERR;
 					fclose (stream);
-					if (!suppressoutput)
-						UIStatus (_("Too many HTTP redirects encountered! Giving up."), 2);
 					return NULL;
 				}
 				
 				while (!feof(stream)) {
 					if ((fgets (netbuf, sizeof(netbuf), stream)) == NULL) {
 						/* Something bad happened. Server sent stupid stuff. */
-						//MainQuit (_("Following HTTP redirects"), strerror(errno));
+						cur_ptr->netio_error = NET_ERR_HTTP_PROTO_ERR;
+						fclose (stream);
 						return NULL;
 					}
 					/* Split netbuf into hostname and trailing url.
@@ -428,8 +463,6 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 					   
 					   Do not touch any of the following code! :P */
 					if (strncasecmp (netbuf, "Location", 8) == 0) {
-						//tmpstring = malloc(strlen(netbuf)+1);
-						
 						redirecttarget = strdup (netbuf);
 						freeme = redirecttarget;
 						
@@ -467,7 +500,8 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 						/* Change cur_ptr->feedurl on 301. */
 						if (cur_ptr->lasthttpstatus == 301) {
 							if (!suppressoutput)
-								UIStatus (_("URL points to permanent redirect, updating with new location..."), 2);
+								UIStatus (_("URL points to permanent redirect, updating with new location..."), 2, 0);
+							printlog (cur_ptr, _("URL points to permanent redirect, updating with new location..."));
 							free (cur_ptr->feedurl);
 							if (authdata == NULL)
 								cur_ptr->feedurl = strdup (newlocation);
@@ -498,8 +532,7 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 						   Say goodbye to the webserver in this case. In fact, we don't
 						   even say goodbye, but just drop the connection. */
 						if (newlocation == NULL) {
-							if (!suppressoutput)
-								UIStatus (_("Webserver sent an invalid redirect!"), 2);
+							cur_ptr->netio_error = NET_ERR_REDIRECT_ERR;
 							fclose (stream);
 							return NULL;
 						}
@@ -515,8 +548,7 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 						fclose (stream);
 						
 						/* Reconnect to server. */
-						if ((NetConnect (my_socket, connectresult, newhost, httpproto)) != 0) {
-							/* Add error handling/reporting. */
+						if ((NetConnect (my_socket, newhost, cur_ptr, httpproto, suppressoutput)) != 0) {
 							return NULL;
 						}
 					
@@ -532,6 +564,7 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 				   Not very friendly though. :) */
 				fclose (stream);
 				/* Received good status from server, clear problem field. */
+				cur_ptr->netio_error = NET_ERR_OK;
 				cur_ptr->problem = 0;
 				
 				/* This should be freed everywhere where we return
@@ -546,12 +579,16 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 				   Parse rest of header and rerequest URL from server using auth mechanism
 				   requested in WWW-Authenticate header field. (Basic or Digest) */
 				break;
+			case 404:
+				cur_ptr->netio_error = NET_ERR_HTTP_404;
+				fclose (stream);
+				return NULL;
 			case 410: /* The feed is gone. Politely remind the user to unsubscribe. */
-				snprintf (tmp, sizeof(tmp), _("The feed \"%s\" no longer exists. Please unsubscribe!"), cur_ptr->title);
-				if (!suppressoutput)
-					UIStatus (tmp, 3);
+				cur_ptr->netio_error = NET_ERR_HTTP_410;
+				fclose (stream);
+				return NULL;
 			case 400:
-				cur_ptr->problem = 1;
+				cur_ptr->netio_error = NET_ERR_HTTP_NON_200;
 				fclose (stream);
 				return NULL;
 			default:
@@ -562,7 +599,8 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 					tmphttpstatus -= tmphttpstatus % 100;
 				} else {
 					/* second pass, give up on unknown error base class */
-					cur_ptr->problem = 1;
+					cur_ptr->netio_error = NET_ERR_HTTP_NON_200;
+					printlog (cur_ptr, servreply);
 					fclose (stream);
 					return NULL;
 				}
@@ -571,25 +609,9 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 	
 	/* Read rest of HTTP header and parse what we need. */
 	while (!feof(stream)) {	
-		while (1) {
-			/* Use select to make the connection interuptable if it should hang.
-			   select() seems to destroy &rfdsw. Put the initialization
-	  		   inside the while() loop. */
-			FD_ZERO(&rfds);
-			FD_SET(0, &rfds);
-			FD_SET(*my_socket, &rfds);
-	   
-			retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
-			
-			if (FD_ISSET (0, &rfds)) {
-				if (getch() == 'z') {
-					fclose (stream);
-					return NULL;
-				}
-			}
-			
-			if (FD_ISSET (*my_socket, &rfds))
-				break;
+		if ((NetPoll (cur_ptr, my_socket, NET_READ)) == -1) {
+			fclose (stream);
+			return NULL;
 		}
 
 		/* Max line length of sizeof(netbuf) is assumed here.
@@ -597,17 +619,10 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 		if ((fgets (netbuf, sizeof(netbuf), stream)) == NULL)
 			break;
 		
-		if (strstr (netbuf, "chunked") != NULL) {
-			/* Server sent junked encoding. Until I understand how it works
-			   and snownews uses HTTP/1.1 we must reject this answer.
-			   Chunked encoding is not defined in HTTP/1.0. */
+		if (strncasecmp (netbuf, "Transfer-Encoding", 17) == 0) {
+			/* Chunked transfer encoding. HTTP/1.1 extension.
+			   http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.6.1 */
 			chunked = 1;
-			snprintf (tmp, sizeof(tmp), _("The webserver %s sent illegal HTTP/1.0 reply! I cannot parse this."), host);
-			if (!suppressoutput)
-				UIStatus (tmp, 2);
-			cur_ptr->problem = 1;
-			fclose (stream);
-			return NULL;
 		}
 		/* Get last modified date. This is only relevant on HTTP 200. */
 		if ((strncasecmp (netbuf, "Last-Modified", 13) == 0) &&
@@ -627,13 +642,9 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 		}
 		/* Check and parse Content-Encoding header. */
 		if (strncasecmp (netbuf, "Content-Encoding", 16) == 0) {
-			/* We do not support deflate. And probably never will.
-			if (strstr (netbuf, "deflate") != NULL)
-				inflate = 1;
-			*/
 			/* Will also catch x-gzip. */
 			if (strstr (netbuf, "gzip") != NULL)
-				inflate = 2;
+				inflate = 1;
 		}		
 		/* HTTP authentication
 		 *
@@ -642,7 +653,7 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 			(cur_ptr->lasthttpstatus == 401)) {
 			if (authfailed) {
 				/* Don't repeat authrequest if it already failed before! */
-				UIStatus (_("Authentication failed!"), 3);
+				cur_ptr->netio_error = NET_ERR_AUTH_FAILED;
 				fclose (stream);
 				return NULL;
 			}
@@ -666,17 +677,17 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 			
 			switch (retval) {
 				case 1:
-					UIStatus (_("URL does not contain authentication information!"), 2);
+					cur_ptr->netio_error = NET_ERR_AUTH_NO_AUTHINFO;
 					fclose (stream);
 					return NULL;
 					break;
 				case 2:
-					UIStatus (_("Could not generate authentication information!"), 2);
+					cur_ptr->netio_error = NET_ERR_AUTH_GEN_AUTH_ERR;
 					fclose (stream);
 					return NULL;
 					break;
 				case -1:
-					UIStatus (_("Unsupported authentication method requested by server!"), 2);
+					cur_ptr->netio_error = NET_ERR_AUTH_UNSUPPORTED;
 					fclose (stream);
 					return NULL;
 					break;
@@ -686,8 +697,7 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 			
 			/* Close current connection and reconnect to server. */
 			fclose (stream);
-			if ((NetConnect (my_socket, connectresult, host, httpproto)) != 0) {
-				/* Add error handling/reporting. */
+			if ((NetConnect (my_socket, host, cur_ptr, httpproto, suppressoutput)) != 0) {
 				return NULL;
 			}
 
@@ -728,25 +738,9 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 
 	/* Read stream until EOF and return it to parent. */
 	while (!feof(stream)) {
-		while (1) {
-			/* Use select to make the connection interuptable if it should hang.
-			   select() seems to destroy &rfdsw. Put the initialization
-	  		   inside the while() loop. */
-			FD_ZERO(&rfds);
-			FD_SET(0, &rfds);
-			FD_SET(*my_socket, &rfds);
-			
-			retval = select (*my_socket+1, &rfds, NULL, NULL, NULL);
-			
-			if (FD_ISSET (0, &rfds)) {
-				if (getch() == 'z') {
-					fclose (stream);
-					return NULL;
-				}
-			}
-			
-			if (FD_ISSET (*my_socket, &rfds))
-				break;
+		if ((NetPoll (cur_ptr, my_socket, NET_READ)) == -1) {
+			fclose (stream);
+			return NULL;
 		}
 		
 		/* Since we handle binary data if we read compressed input we
@@ -762,40 +756,34 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
 	}
 	body = realloc(body, length+1);
 	body[length] = '\0';
-		
+	
+	cur_ptr->contentlength = length;
+	
 	/* Close connection. */
 	fclose (stream);
 	
-	/* If inflate==1 we need to decompress content with deflate.
-	   Probably not needed since every webserver seems to send gzip. */
-	if (inflate == 1) {
-		inflatedbody = zlib_uncompress (body, length, NULL, 0);
-		
-		if (inflatedbody == NULL) {
+	if (chunked) {
+		if (decodechunked(body, &length) == NULL) {
 			free (body);
+			cur_ptr->netio_error = NET_ERR_HTTP_PROTO_ERR;
 			return NULL;
 		}
-		
-		/* Copy uncompressed data back to body. */
-		free (body);
-		body = strdup (inflatedbody);
-		free (inflatedbody);
-	} else if (inflate == 2) {
-		/* gzipinflate */
-		inflatedbody = gzip_uncompress (body, length, NULL);
-		
-		if (inflatedbody == NULL) {
-			free (body);
-			return NULL;
-		}
-		
-		/* Copy uncompressed data back to body. */
-		free (body);
-		body = strdup (inflatedbody);
-		free (inflatedbody);
 	}
 	
-	cur_ptr->contentlength = strlen (body);
+	/* If inflate==1 we need to decompress the content.. */
+	if (inflate == 1) {
+		/* gzipinflate */
+		inflatedbody = gzip_uncompress (body, length, &cur_ptr->contentlength);
+		if (inflatedbody == NULL) {
+			free (body);
+			cur_ptr->netio_error = NET_ERR_GZIP_ERR;
+			return NULL;
+		}
+		
+		/* Copy uncompressed data back to body. */
+		free (body);
+		body = inflatedbody;
+	}
 	
 	return body;
 }
@@ -805,15 +793,13 @@ char * NetIO (int * my_socket, int * connectresult, char * host, char * url, str
    Set suppressoutput=1 to disable ncurses calls. */
 char * DownloadFeed (char * url, struct feed * cur_ptr, int suppressoutput) {
 	int my_socket = 0;
-	int connectresult = 0;
-	int result;
+	int url_fixup = 0;
 	char *host;					/* Needs to freed. */
 	char *tmphost;
 	char *freeme;
 	char *returndata;
 	char *authdata = NULL;
 	char *tmpstr;
-	char tmp[1024];
 	int httpproto = 0;			/* 0: http; 1: https */
 	
 	/* strstr will match _any_ substring. Not good, use strncasecmp with length 5! */
@@ -822,25 +808,14 @@ char * DownloadFeed (char * url, struct feed * cur_ptr, int suppressoutput) {
 	else
 		httpproto = 0;
 	
-	/* hahteeteepeh://foo.bar/jaaaaaguar.html.
-	                00^      0^
-	
-	   Warning!
-	   This "function" will not handle the following URLs:
-	   http://foo.bar/
-	   
-	   Must add index.rdf or whatever. */
 	strsep (&url, "/");
 	strsep (&url, "/");
 	tmphost = url;
 	strsep (&url, "/");
 	if (url == NULL) {
-		/* End of string url. Probably entered invalid stuff like
-		   http://asdf or something. Since we don't want to crash and burn
-		   return the dreaded NULL pointer. */
-		if (!suppressoutput)
-			UIStatus (_("This URL doesn't look too valid to me, don't you think?"), 2);
-		return NULL;
+		/* Assume "/" is input is exhausted. */
+		url = strdup("/");
+		url_fixup = 1;
 	}
 	
 	/* If tmphost contains an '@', extract username and pwd. */
@@ -855,59 +830,35 @@ char * DownloadFeed (char * url, struct feed * cur_ptr, int suppressoutput) {
 	/* netio() might change pointer of host to something else if redirect
 	   loop is executed. Make a copy so we can correctly free everything. */
 	freeme = host;
-	url--;
-	url[0] = '/';
-	
-	if (url[strlen(url)-1] == '\n') {
-		url[strlen(url)-1] = '\0';
+	/* Only run if url was != NULL above. */
+	if (!url_fixup) {
+		url--;
+		url[0] = '/';
+		if (url[strlen(url)-1] == '\n') {
+			url[strlen(url)-1] = '\0';
+		}
 	}
 	
-	result = NetConnect (&my_socket, &connectresult, host, httpproto);
-	
-	switch (result) {
-		case 1:
-			if (!suppressoutput)
-				UIStatus (_("Couldn't create network socket!"), 2);
-			cur_ptr->problem = 1;
-			free (freeme);
-			free (authdata);
-			return NULL;
-		case 2:
-			snprintf (tmp, sizeof(tmp), _("Can't resolve host %s!"), host);
-			if (!suppressoutput)
-				UIStatus (tmp, 2);
-			cur_ptr->problem = 1;
-			free (freeme);
-			free (authdata);
-			return NULL;
-		case 3:
-			/* On Solaris connect() might return -1, passing -1 to strerror()
-			   will return a NULL pointer which will segfault snprintf().
-			   This is broken IMO, but the check catches the crash. */
-			snprintf (tmp, sizeof(tmp), _("Could not connect to server %s: %s"), host,
-				(strerror(connectresult) ? strerror(connectresult) : "(null)"));
-			if (!suppressoutput)
-				UIStatus (tmp, 2);
-			cur_ptr->problem = 1;
-			free (freeme);
-			free (authdata);
-			return NULL;
-		case -1:
-			if (!suppressoutput)
-				UIStatus (_("Aborted."), 2);
-			free (freeme);
-			free (authdata);
-			return NULL;
-		default:
-			break;
+	if ((NetConnect (&my_socket, host, cur_ptr, httpproto, suppressoutput)) != 0) {
+		free (freeme);
+		free (authdata);
+		if (url_fixup)
+			free(url);
+		cur_ptr->problem = 1;
+		return NULL;
 	}
+		
+	returndata = NetIO (&my_socket, host, url, cur_ptr, authdata, httpproto, suppressoutput);
 	
-	returndata = NetIO (&my_socket, &connectresult, host, url, cur_ptr, authdata, httpproto, suppressoutput);
+	if ((returndata == NULL) && (cur_ptr->netio_error != NET_ERR_OK)) {
+		cur_ptr->problem = 1;
+	}
 	
 	/* url will be freed in the calling function. */
 	free (freeme);		/* This is *host. */
-	
 	free (authdata);
+	if (url_fixup)
+		free(url);
 	
 	return returndata;
 }
