@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "main.h"
 
@@ -37,9 +38,8 @@
  *
  * http://www.winehq.com/hypermail/wine-patches/2001/11/0024.html
  *
- * The following function is written by Franois Gouget
+ * The following function was written by Franois Gouget.
  */
-
 #ifdef SUN
 char* strsep(char** str, const char* delims)
 {
@@ -65,6 +65,20 @@ char* strsep(char** str, const char* delims)
 }
 #endif
 
+/* strcasestr stolen from: http://www.unixpapa.com/incnote/string.html */
+char *s_strcasestr(char *a, char *b) {
+	size_t l;
+	char f[3];
+	int lena = strlen(a);
+	int lenb = strlen(b);
+	
+	snprintf(f, sizeof(f), "%c%c", tolower(*b), toupper(*b));
+	for (l = strcspn(a, f); l != lena; l += strcspn(a + l + 1, f) + 1)
+		if (strncasecmp(a + l, b, lenb) == 0)
+			return(a + l);
+	return(NULL);
+}
+
 
 /* Private malloc wrapper. Aborts program execution if malloc fails. */
 void * s_malloc (size_t size) {
