@@ -61,7 +61,7 @@ void SetupBrowser (char * file) {
 
 	configfile = fopen (file, "r");
 	if (configfile == NULL) {
-		UIStatus (_("Creating new config \"browser\"..."), 0);
+		UIStatus (_("Creating new config \"browser\"..."), 0, 0);
 		configfile = fopen (file, "w+");
 		if (configfile == NULL)
 			MainQuit (_("Create initial configfile \"config\""), strerror(errno)); /* Still didn't work? */
@@ -123,7 +123,10 @@ void SetupProxy (void) {
    This includes Snownews/VERSION, OS name and language setting. */
 void SetupUserAgent (void) {
 	char *lang;
-	int ualength;
+	char url[] = "http://snownews.kcore.de/";
+	int ualen, urllen;
+
+	urllen = strlen(url);
 	
 	/* Constuct the User-Agent string of snownews. This is done here in program init,
 	   because we need to do it exactly once and it will never change while the program
@@ -131,14 +134,14 @@ void SetupUserAgent (void) {
 	if (getenv("LANG") != NULL) {
 		lang = getenv("LANG");
 		/* Snonews/VERSION (Linux; de_DE; (http://kiza.kcore.de/software/snownews/) */
-		ualength = strlen("Snownews/") + strlen(VERSION) + 2 + strlen(lang) + 2 + strlen(OS)+2 + strlen("http://kiza.kcore.de/software/snownews/") + 2;
-		useragent = malloc(ualength);
-		snprintf (useragent, ualength, "Snownews/%s (%s; %s; http://kiza.kcore.de/software/snownews/)", VERSION, OS, lang);
+		ualen = strlen("Snownews/") + strlen(VERSION) + 2 + strlen(lang) + 2 + strlen(OS)+2 + urllen + 2;
+		useragent = malloc(ualen);
+		snprintf (useragent, ualen, "Snownews/%s (%s; %s; %s)", VERSION, OS, lang, url);
 	} else {
 		/* "Snownews/" + VERSION + "(http://kiza.kcore.de/software/snownews/)" */
-		ualength = strlen("Snownews/") + strlen(VERSION) + 2 + strlen(OS) + 2 + strlen("http://kiza.kcore.de/software/snownews/") + 2;
-		useragent = malloc(ualength);
-		snprintf (useragent, ualength, "Snownews/%s (%s; http://kiza.kcore.de/software/snownews/)", VERSION, OS);
+		ualen = strlen("Snownews/") + strlen(VERSION) + 2 + strlen(OS) + 2 + urllen + 2;
+		useragent = malloc(ualen);
+		snprintf (useragent, ualen, "Snownews/%s (%s; %s)", VERSION, OS, url);
 	}
 }
 
@@ -153,6 +156,7 @@ void SetupKeybindings (char * file) {
 	   these are sane defaults. */
 	keybindings.next = 'n';
 	keybindings.prev = 'p';
+	keybindings.prevmenu = 'q';
 	keybindings.quit = 'q';
 	keybindings.addfeed = 'a';
 	keybindings.deletefeed = 'D';
@@ -171,8 +175,8 @@ void SetupKeybindings (char * file) {
 	keybindings.sortfeeds = 's';
 	keybindings.pup = 'b';
 	keybindings.pdown = ' ';
-	keybindings.home = 'w';
-	keybindings.end = 'W';
+	keybindings.home = '<';
+	keybindings.end = '>';
 	keybindings.categorize = 'C';
 	keybindings.filter = 'f';
 	keybindings.filtercurrent = 'g';
@@ -205,7 +209,9 @@ void SetupKeybindings (char * file) {
 				keybindings.next = tmpbuf[0];
 			else if (strcmp (freeme, "previous item") == 0)
 				keybindings.prev = tmpbuf[0];
-			else if (strcmp (freeme, "quit/return to previous menu") == 0)
+			else if (strcmp (freeme, "return to previous menu") == 0)
+				keybindings.prevmenu = tmpbuf[0];
+			else if (strcmp (freeme, "quit") == 0)
 				keybindings.quit = tmpbuf[0];
 			else if (strcmp (freeme, "add feed") == 0)
 				keybindings.addfeed = tmpbuf[0];
@@ -295,6 +301,7 @@ void SetupKeybindings (char * file) {
 	fprintf (configfile, "remove filter:%c\n", keybindings.nofilter);
 	fprintf (configfile, "per feed filter:%c\n", keybindings.perfeedfilter);
 	fprintf (configfile, "toggle AND/OR filtering:%c\n", keybindings.andxor);
+	fprintf (configfile, "quit:%c\n", keybindings.quit);
 	fputs ("# Feed display menu bindings\n", configfile);
 	fprintf (configfile, "show feedinfo:%c\n", keybindings.feedinfo);
 	fprintf (configfile, "mark feed as read:%c\n", keybindings.markread);
@@ -302,7 +309,7 @@ void SetupKeybindings (char * file) {
 	fputs ("# General keybindungs\n", configfile);
 	fprintf (configfile, "next item:%c\n", keybindings.next);
 	fprintf (configfile, "previous item:%c\n", keybindings.prev);
-	fprintf (configfile, "quit/return to previous menu:%c\n", keybindings.quit);
+	fprintf (configfile, "return to previous menu:%c\n", keybindings.prevmenu);
 	fprintf (configfile, "reload feed:%c\n", keybindings.reload);
 	fprintf (configfile, "open url:%c\n", keybindings.urljump);
 	fprintf (configfile, "open item url in overview:%c\n", keybindings.urljump2);
@@ -567,7 +574,7 @@ int Config (void) {
 			MainQuit (_("Creating config directory ~/.snownews/cache/"), strerror(errno));
 	} else {
 		if ((dirtest.st_mode & S_IFDIR) != S_IFDIR) {
-			MainQuit ("Creating config directory ~/.snownews/cache",
+			MainQuit ("Creating config directory ~/.snownews/cache/",
 				"A file with the name \"~/.snownews/cache/\" exists!");
 		}
 	}
@@ -578,7 +585,7 @@ int Config (void) {
 	errorlog = fopen (file, "w+");
 	dup2 (fileno(errorlog), STDERR_FILENO);
 	
-	UIStatus (_("Reading configuration settings..."), 0);
+	UIStatus (_("Reading configuration settings..."), 0, 0);
 	
 	snprintf (file, sizeof(file), "%s/.snownews/browser", getenv("HOME"));
 	SetupBrowser (file);
@@ -590,7 +597,7 @@ int Config (void) {
 	snprintf (file, sizeof(file), "%s/.snownews/urls", getenv("HOME"));
 	configfile = fopen (file, "r");
 	if (configfile == NULL) {
-		UIStatus (_("Creating new configfile."), 0);
+		UIStatus (_("Creating new configfile."), 0, 0);
 		configfile = fopen (file, "w+");
 		if (configfile == NULL)
 			MainQuit (_("Create initial configfile \"urls\""), strerror(errno));	/* Still didn't work? */
@@ -611,6 +618,8 @@ int Config (void) {
 			new_ptr->description = NULL;
 			new_ptr->lastmodified = NULL;
 			new_ptr->lasthttpstatus = 0;
+			new_ptr->netio_error = NET_ERR_OK;
+			new_ptr->connectresult = 0;
 			new_ptr->cookies = NULL;
 			new_ptr->authinfo = NULL;
 			new_ptr->servauth = NULL;
