@@ -24,7 +24,7 @@
 #define UI_SUPPORT_H
 
 void InitCurses (void);
-void UIStatus (char * text, int delay);
+void UIStatus (char * text, int delay, int warning);
 void SwapPointers (struct feed * one, struct feed * two);
 void SnowSort (void);
 void UISupportDrawBox (int x1, int y1, int x2, int y2);
@@ -32,5 +32,7 @@ void UISupportDrawHeader (char * headerstring);
 void UISupportURLJump (char * url);
 void SmartFeedsUpdate (void);
 void SmartFeedNewitems (struct feed * smart_feed);
+int SmartFeedExists (const char * smartfeed);
+void DrawProgressBar (int numobjects, int titlestrlen);
 
 #endif
