@@ -238,12 +238,18 @@ void parse_rdf10_item(struct feed *feed, xmlDocPtr doc, xmlNodePtr node)
 	if (saverestore == 1) {
 		for (current = firstcopy; current != NULL; current = current->next_ptr) {
 			if ((current->data->link != NULL) && (item->data->link != NULL)) {
-				if (strcmp(item->data->link, current->data->link) == 0)
-					item->data->readstatus = current->data->readstatus;
+				if ((current->data->title != NULL) && (item->data->title != NULL)) {
+					if ((strcmp(item->data->link, current->data->link) == 0) &&
+						(strcmp(item->data->title, current->data->title) == 0))
+						item->data->readstatus = current->data->readstatus;
+				} else {
+					if (strcmp(item->data->link, current->data->link) == 0)
+						item->data->readstatus = current->data->readstatus;
+				}
 			}
 		}
 	}
-		
+
 	item->next_ptr = NULL;
 	if (feed->items == NULL) {
 		item->prev_ptr = NULL;
@@ -277,12 +283,13 @@ int DeXML (struct feed * cur_ptr) {
 			copy = malloc (sizeof(struct newsitem));
 			copy->data = malloc (sizeof (struct newsdata));
 			copy->data->title = NULL;
+			copy->data->link = NULL;
 			copy->data->description = NULL;
 			copy->data->readstatus = cur_item->data->readstatus;
 			if (cur_item->data->link != NULL)
 				copy->data->link = strdup (cur_item->data->link);
-			else
-				copy->data->link = NULL;
+			if (cur_item->data->title != NULL)
+				copy->data->title = strdup (cur_item->data->title);
 			
 			copy->next_ptr = NULL;
 			if (firstcopy == NULL) {
@@ -353,10 +360,12 @@ int DeXML (struct feed * cur_ptr) {
 		while (firstcopy->next_ptr != NULL) {
 			firstcopy = firstcopy->next_ptr;
 			free (firstcopy->prev_ptr->data->link);
+			free (firstcopy->prev_ptr->data->title);
 			free (firstcopy->prev_ptr->data);
 			free (firstcopy->prev_ptr);
 		}
 		free (firstcopy->data->link);
+		free (firstcopy->data->title);
 		free (firstcopy->data);
 		free (firstcopy);
 	}
